-- main.lua - a test script for line segment interception

function love.load()
    color = {
        white	= {255,255,255,255},
        black	= {  0,  0,  0,255},
        gray	= {128,128,128,255},
        window 	= {  0, 80,160,255},
        shadow	= { 96, 96, 96,255},
		red		= {255,  0,  0,255},
		green	= {  0,255,  0,255},
		yellow	= {255,255,  0,255},
		vred	= {255,  0,  0,255},
		blue	= { 50, 50,255,255}
		}
	gr = love.graphics
	gr.setBackgroundColor( 15, 15, 15, 55)
	width, height = gr.getWidth(), gr.getHeight()
	col = {{255,  0,  0},{255,155,  0},{255,225,125},{155,  0,155},{155,155,155},
		   {155,155,255},{  0,155,  0},{  0,155,155},{  0,255,155},{  0,  0,255}}
	object = {				-- object being probed
		sides	= 3,
		x		= 290,		-- center of object
		y		= 290,
		size	= 135}
	line = figure( object )	-- table of lines defining object
	rayAngle, rayLength = 0, 190
	rx1, ry1, rx2, ry2 = rotateRay( rayAngle, rayLength )
	end

function love.update(dt)
	rayAngle			= rayAngle + dt
	if rayAngle			>= 2 * math.pi then
		rayAngle 		= 0
		object.sides	= object.sides < 10 and object.sides + 1 or 3
		line			= figure( object )
		end
	rx1, ry1, rx2, ry2	= rotateRay( rayAngle, rayLength )
	end

function love.draw()
	gr.setColor( color.gray )
	gr.line( 0, 200, 400, 200 )
	gr.line( 200, 0, 200, 400 )
	gr.setColor(color.white)
	gr.circle( "fill", 10, 15, 5)
	gr.print( "contact points", 20,10)
	--gr.setColor( color.red )
	gr.circle( "line",10,30,5)
	gr.print("intersects off segments",20,25)
	-- draw object
	for i = 1, #line do gr.setColor(col[i]) gr.line( unpack( line[ i ] ) ) end
	-- draw ray
	gr.setColor( color.green )
	gr.line( rx2, ry2, rx1, ry1 )
	-- see if ray strikes object
	for i = 1, #line do checkForContact( i ) end
	end

function love.keypressed(k)
	if k == 'escape' then love.event.push( 'quit' )	end
	end

-- =====================================================
function figure( tab )	-- 2D figure with specified # of sides
	-- tab contains number of sides, location of center & object size
	local angle_step	= math.pi*360 / (tab.sides*180)
	local shape			= {}
	for side			= 1, tab.sides do
		shape[side]		= {}
		shape[side][1]	= tab.size*math.sin( (angle_step*(side-1)) ) / 2+tab.x
		shape[side][2]	= tab.size*math.cos( (angle_step*(side-1)) ) / 2+tab.y
		shape[side][3]	= tab.size*math.sin( (angle_step*(side))   ) / 2+tab.x
		shape[side][4]	= tab.size*math.cos( (angle_step*(side))   ) / 2+tab.y
		end
	return shape		-- a table of line segments
	end
-- =====================================================
function rotateRay( angle, length )
	local x1 = length * math.sin( angle ) + width / 2
	local y1 = length * math.cos( angle ) + height / 2
	local x2 = width / 2
	local y2 = height / 2
	return x1, y1, x2, y2
	end
-- ======================================================
function checkForContact( num )
	contact, x, y = segmentIntersection( rx1, ry1, rx2, ry2, unpack( line[ num ] ) )
	if contact then
		gr.setColor( color.yellow )
		gr.setColor(col[num])
		gr.circle( "fill", x, y, 5)
		msg = "line "..num.." contact at "..math.floor(x)..", "..math.floor(y)
		gr.print( msg, 10, height-(#line * 15 - ( num - 1 ) * 15) )
		else
		gr.setColor(col[num])
		gr.circle( "line",x, y, 5)
		gr.print( "no contact with line "..num, 10, height - (#line * 15 - ( num - 1 ) * 15) )
		end
	end
-- ======================================================
function segmentIntersection(s1x1, s1y1, s1x2, s1y2, s2x1, s2y1, s2x2, s2y2)
	-- compensate for horizontal & verticle lines
	local eps = 0.0001
	if s1x1 == s1x2 then s1x2 = s1x2 + eps end
	if s1y1 == s1y2 then s1y2 = s1y2 + eps end
	if s2x1 == s2x2 then s2x2 = s2x2 + eps end
	if s2y1 == s2y2 then s2y2 = s2y2 + eps end
	-- fit linear equation to line segments
	local slope1 = ( s1y2 - s1y1 ) / ( s1x2 - s1x1 )	-- line1 slope
	local slope2 = ( s2y2 - s2y1 ) / ( s2x2 - s2x1 )	-- line2 slope
	local int1	 = s1y1 - slope1 * s1x1					-- line1 intercept
	local int2	 = s2y1 - slope2 * s2x1					-- line2 intercept
	-- intercept coordinates
	if slope1 == slope2 then slope1 = slope1 + eps end	-- avoid div by zero
	local x = ( int2 - int1 ) / ( slope1 - slope2 )
	local y = slope2 * x + int2
	-- check order of points
	if s1x1 > s1x2 then s1x1, s1x2 = s1x2, s1x1 end
	if s1y1 > s1y2 then s1y1, s1y2 = s1y2, s1y1 end
	if s2x1 > s2x2 then s2x1, s2x2 = s2x2, s2x1 end
	if s2y1 > s2y2 then s2y1, s2y2 = s2y2, s2y1 end
	-- check if contact point is on both line segments
	if  x >= s1x1 and x <= s1x2 and x >= s2x1 and x <= s2x2 and
		y >= s1y1 and y <= s1y2 and y >= s2y1 and y <= s2y2 then
		return true, x, y
		else
		return false, x, y
		end
	end
